/*--------------------------------------------------------------------
 * FICHERO:  PerfRTF.c
 * OBJETIVO: Convertir un fichero Perfection SE 4.12 a Rich Text Format
 * AUTOR:    Pedro Reina
 * FECHA:    M.3.3.1998
 *------------------------------------------------------------------*/

/*--------------------------------------------------------------------
 * Ficheros de cabecera
 *------------------------------------------------------------------*/

#include <Olimpo.h>            /*  Versin 2.0  */
#include <string.h>            /*  strtok()     */
#include "Estilo.h"
#include "Perfectn.h"

/*--------------------------------------------------------------------
 * Declaracin de funciones
 *------------------------------------------------------------------*/

void   LeeConfig();
logico AlmacenaFuente();

void Explica();
void ExplicaObjetivo();
void ExplicaPrograma();
void ExplicaAutor();

void   Fichero();
cadena EligeUnNombre();
cadena PideDirectorio();
cadena PideExtension();
cadena PideMascara();
cadena PideNombre();
cadena NombreCompleto();
void   TrataFichero();
logico CompruebaPerfection();
void   Convierte();
octeto CalculaAsignacionFuente();
void   HallaInfoEstilo();
void   ExplicaEstado();

void     GestionaEstilo();
void     LeeFicheroEstilo();
void     GrabaFicheroEstilo();
void     DefineTablaEstilo();
estilo   DefineEstilo();
cadena   PideNombreEstilo();
contador PideFuenteEstilo();
octeto   PideJustificacionEstilo();
contador PideMargenEstilo();
octeto   PideTamanoEstilo();

void   Opciones();
octeto PideTamano();
octeto PidePosicion();

void Rtf_EscribeCadena();
void Rtf_EscribeEstilo();
void Rtf_NuevaLinea();
void Rtf_EscribeCaracter();

/*--------------------------------------------------------------------
 * Definicin de macros constantes
 *------------------------------------------------------------------*/

#define MAX_ESTILO 8

#define LONG_DIRECTORIO  24
#define LONG_EXTENSION    3
#define LONG_NOMBRE      12
#define LONG_MASCARA     12
#define LONG_DESTINO     20

#define LONG_NOMBRE_EST  15

#define MAX_FICHERO     200
#define MAX_FUENTE      200

/*--------------------------------------------------------------------
 * Definicin de macros funciones
 *------------------------------------------------------------------*/

#define LimpiaZonaTrabajo()     Pan_Borra(NEGRO,4,0,21,79)
#define LimpiaZonaPequena()     Pan_Borra(NEGRO,10,0,21,79)
#define LimpiaZonaEstilo()      Pan_Borra(NEGRO,12,0,21,79)

/*--------------------------------------------------------------------
 * MACRO:    Twip()
 * OBJETIVO: Pasar de milmetros a twips
 * ENTRADAS: La cantidad en milmetros
 * SALIDAS:  La cantidad en twips
 * NOTAS:    Un twip es la vigsima parte de un punto tipogrfico
 *           Un punto tipogrfico es 1/72 pulgadas
 *           Una pulgada es 25.4 milmetros
 *------------------------------------------------------------------*/
#define Twip(x)  ((contador)((x)*56.692913+0.5))

/*--------------------------------------------------------------------
 * MACROS DE MANEJO DE RESALTADO DE LETRA
 * OBJETIVO: Manejar las distintas maneras de resaltar la letra
 * ENTRADAS: Un octeto con el estado actual de resaltado
 * SALIDAS:  El nuevo estado o bien la respuesta a la consulta
 * NOTAS:    Cada bit del octeto maneja un modo de resaltado:
 *           Bit 0 -> Negrita
 *           Bit 1 -> Itlica
 *           Bit 2 -> Versalita
 *           Bit 3 -> Subndice
 *           Bit 4 -> Superndice
 *------------------------------------------------------------------*/

#define Negrita(e)       (((e)&1)>0)
#define PonNegrita(e)    ((e)|=(octeto)1)
#define QuitaNegrita(e)  ((e)&=(octeto)254)

#define Italica(e)       (((e)&2)>0)
#define PonItalica(e)    ((e)|=(octeto)2)
#define QuitaItalica(e)  ((e)&=(octeto)253)

#define Versalita(e)       (((e)&4)>0)
#define PonVersalita(e)    ((e)|=(octeto)4)
#define QuitaVersalita(e)  ((e)&=(octeto)251)

#define Subindice(e)       (((e)&8)>0)
#define PonSubindice(e)    ((e)|=(octeto)8)
#define QuitaSubindice(e)  ((e)&=(octeto)247)

#define Superindice(e)       (((e)&16)>0)
#define PonSuperindice(e)    ((e)|=(octeto)16)
#define QuitaSuperindice(e)  ((e)&=(octeto)239)

/*--------------------------------------------------------------------
 * Definicin de variables globales
 *------------------------------------------------------------------*/

cadena ExtensionPerfection[4];
cadena FicheroEstilo = CAD_FIN;

estilo   Estilo[MAX_ESTILO];
cadena   FuenteDisponible[MAX_FUENTE];
octeto   TipoFuente[MAX_FUENTE];
cadena   Digito = "0123456789ABCDEF";
octeto   CarExt[4] = {92, 39, 0, 0};   /*  \'00  */
contador TotalFuente;
octeto   TamanoSubSuper=58;
octeto   PosicionSubSuper=33;

/*--------------------------------------------------------------------
 * Programa principal
 *------------------------------------------------------------------*/
void main (Narg, Arg)
int   Narg;
char *Arg[];
  {
  enum { SALIDA_ESC, SALIDA, EXPLICA, FICHERO, ESTILO, OPCIONES };

  static cadena Principal[] = {">Salida", "E>xplica", ">Fichero", ">Estilo",
                               ">Opciones", NIL };
  entero   Opcion;
  contador NumEstilo;
  logico   Sigue=SI;
  cadena   Nombre;

  Pan_Define (PAN_TEXTO);
  Prg_Presenta ("Perfection a RTF", "1.1", "Pedro Reina", "Marzo 1998");
  Cdr_Caja (CDR_SIMPLE,1,0,3,79,NEGRO,VERDE);

  Cad_Copia (ExtensionPerfection,"prf");
  for ( NumEstilo = 0 ; NumEstilo < MAX_ESTILO ; NumEstilo++ )
    { Estilo[NumEstilo] = Est_Crea(); }

  if ( Narg == 1 )
    { Nombre = Fch_Nombre ( "PerfRTF","cnf"); }
  else
    { Nombre = Cad_Duplica ( Arg[1] ); }

  if ( Fch_Existe (Nombre) ) { LeeConfig (Nombre); }
  else
    { Usr_ErrorFatal ("Error fatal: falta fichero de configuracin."); }
  Cad_Destruye (Nombre);

  if ( TotalFuente )
    { FuenteDisponible[TotalFuente] = CAD_FIN; }
  else
    { Usr_ErrorFatal ("Error fatal: no hay ninguna fuente disponible."); }

  if ( FicheroEstilo )  { LeeFicheroEstilo (FicheroEstilo); }

  Opcion = FICHERO;
  while ( Sigue )
    {
    Opcion = Men_Horizontal (2,2,77,Principal,Opcion);
    switch ( Opcion )
      {
      case SALIDA_ESC:
      case SALIDA:
           if ( Usr_Consulta ("Quieres terminar el programa?") )
             { Sigue = NO; }
           Opcion = SALIDA;
           break;
      case EXPLICA:    Explica();          break;
      case FICHERO:    Fichero();          break;
      case ESTILO:     GestionaEstilo();   break;
      case OPCIONES:   Opciones();         break;
      }
    LimpiaZonaTrabajo();
    }

  for ( NumEstilo = 0 ; NumEstilo < MAX_ESTILO ; NumEstilo++ )
    { Est_Destruye (Estilo[NumEstilo]); }

  Pan_Cierra();
  }

/*--------------------------------------------------------------------
 * Definicin de funciones
 *------------------------------------------------------------------*/

/*--------------------------------------------------------------------
 * FUNCION:  Explica()
 * OBJETIVO: Dirigir las explicaciones
 *------------------------------------------------------------------*/
void Explica()
  {
  enum { SALIDA_ESC, OBJETIVO, PROGRAMA, AUTOR };
  static contador MaxOpcion = 3;

  static cadena Menu[] = {">Objetivo", ">Programa", ">Autor", NIL };

  static cadena Presentacion[] = {
    "Puedes pedir explicacin sobre diferentes aspectos",
    "de este programa. Elige el que desees y pulsa ESC",
    "cuando hayas terminado.",
    NIL };

  logico   Sigue = SI;
  entero   Opcion = 1;
  contador i;

  while ( Sigue )
    {
    LimpiaZonaTrabajo();
    Pan_Color (NEGRO,BLANCO);
    for ( i=0 ; Presentacion[i] ; i++ )
      { Pan_PonTexto (6+i,15,Presentacion[i]); }

    Opcion = Men_Vertical (11,34,14,50,Menu,Opcion);
    LimpiaZonaTrabajo();
    switch ( Opcion )
      {
      case SALIDA_ESC:  Sigue = NO;         break;
      case OBJETIVO:    ExplicaObjetivo();  break;
      case PROGRAMA:    ExplicaPrograma();  break;
      case AUTOR:       ExplicaAutor();     break;
      }
    Opcion++;
    if ( Opcion > MaxOpcion )  { Opcion = 1; }
    }
  }

/*--------------------------------------------------------------------
 * FUNCION:  ExplicaProblema()
 * OBJETIVO: Explicar en qu consiste el problema
 *------------------------------------------------------------------*/
void ExplicaObjetivo()
  {
  static cadena Explicacion[] = {
    "Este programa convierte un fichero Perfection SE (de Digital",
    "Precision Ltd.) en un fichero Rich Text Format (RTF), que se",
    "puede importar directamente en cualquier programa que maneje",
    "correctamente RTF. Cada strip de Perfection se convierte en",
    "un estilo de prrafo; se conserva la letra negrita, la",
    "itlica, los subndices y superndices; la letra subrayada",
    "se convierte en versalita.",
    "",
    "El programa se puede configurar desde un fichero y las",
    "definiciones de estilos se pueden leer y grabar en ficheros,",
    "as como definir desde el programa.",
    "",
    "El usuario debe saber el nombre y el tipo de las fuentes que",
    "usar en el programa receptor, para lo cual puede escribir un",
    "pequeo texto con l, exportarlo como RTF y consultar el",
    "grupo \\fonttbl",
    NIL };

  contador i;

  Pan_Color (NEGRO,BLANCO);
  for ( i=0 ; Explicacion[i] ; i++ )
    { Pan_PonTexto (5+i,10,Explicacion[i]); }

  Usr_PulsaUnaTecla ("");
  }

/*--------------------------------------------------------------------
 * FUNCION:  ExplicaPrograma()
 * OBJETIVO: Explicar el funcionamiento del programa
 *------------------------------------------------------------------*/
void ExplicaPrograma()
  {
  static cadena Explicacion[] = {
    "El programa est dirigido por mens. El principal es el",
    "men horizontal que siempre est presente en la parte",
    "superior de la pantalla.",
    "",
    "Con la opcin Fichero puedes convertir un fichero de",
    "Perfection a RTF. Con la opcin Estilo puedes leer y",
    "definir estilos y grabarlos en ficheros.",
    "",
    "En Opciones se pueden definir tres cosas: si el sonido",
    "est encendido o apagado y el tamao y la posicin de las",
    "letras que vayan en subndice o superndice, que se mide",
    "porcentualmente en relacin con la letra base.",
    "",
    "Para ms informacin lee el manual en el fichero TXT.",
    NIL };

  contador i;

  Pan_Color (NEGRO,BLANCO);
  for ( i=0 ; Explicacion[i] ; i++ )
    { Pan_PonTexto (5+i,10,Explicacion[i]); }

  Usr_PulsaUnaTecla ("");
  }

/*--------------------------------------------------------------------
 * FUNCION:  ExplicaAutor()
 * OBJETIVO: Explicar quin es el autor y cmo contactar con l
 *------------------------------------------------------------------*/
void ExplicaAutor()
  {
  static cadena Mensaje[] = {
    "Este programa es de Dominio Pblico. Lo puedes usar, distribuir",
    "y modificar como desees.",
    "",
    "Ha sido escrito en C usando el sistema de programacin Olimpo;",
    "si no dispones de la versin 2.0 de Olimpo, puedes cargarla",
    "directamente desde mi sede web o pedrmela si no dispones de",
    "acceso a Internet.",
    "",
    "Si tienes cualquier duda o consulta, cuntamela e intentar ayudarte.",
    "",
    "Mis datos:",
    "",
    "Pedro Reina",
    "c/ Marquesa de Argeso, 4      Telfono: 91 565 17 59",
    "28019 Madrid                   Correo electrnico: pedro@anit.es",
    "Espaa                         Web: www.anit.es/pedro",
    NIL };

  octeto i;

  Pan_Color (NEGRO,BLANCO);
  for ( i=0 ; Mensaje[i] ; i++ )
    { Pan_PonTexto (5+i,5,Mensaje[i]); }

  Usr_PulsaUnaTecla ("");
  }

/*--------------------------------------------------------------------
 * FUNCION:  Fichero()
 * OBJETIVO: Dirigir la conversin de ficheros
 * ENTRADAS: Ninguna
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void Fichero()
  {
  enum { SALIDA_ESC, CONVIERTE, DIRECTORIO, MASCARA, EXTENSN };

  static cadena MenuFichero[] = {">Convierte", ">Directorio",
                                 ">Mscara", ">Extensin", NIL };

  static char Directorio [LONG_DIRECTORIO+1];
  static char Nombre [LONG_NOMBRE+1];
  static char Mascara [LONG_MASCARA+1] = "*";
  static char Extension [LONG_EXTENSION+1];
  static logico Primero=SI;

  cadena   Aux;
  entero   Opcion;
  contador Fil=4, Col=20, Ancho=40, Alto=5;
  logico   Sigue = SI, Refresca = SI;

  if ( Primero )
    {
    Cad_Copia (Extension, ExtensionPerfection);
    Primero = NO;
    }

  Cdr_Caja (CDR_SIMPLE,Fil,Col,Fil+Alto,Col+Ancho,NEGRO,VERDE);

  Opcion = CONVIERTE;
  while ( Sigue )
    {
    if ( Refresca )
      {
      Pan_Borra (NEGRO,Fil+2,Col+15,Fil+Alto-1,Col+Ancho-1);
      Pan_Color (NEGRO,VERDE);
      Pan_PonTexto (Fil+2,Col+15,Directorio);
      Pan_PonTexto (Fil+3,Col+15,Mascara);
      Pan_PonTexto (Fil+4,Col+15,Extension);
      Refresca = NO;
      }

    Opcion = Men_Vertical (Fil+1,Col+1,Fil+Alto-1,Col+13,MenuFichero,Opcion);
    switch ( Opcion )
      {
      case SALIDA_ESC:  Sigue = NO;      break;

      case CONVIERTE:   Aux = EligeUnNombre (Directorio, Mascara,
                                             Extension, 0);
                        if ( Aux )
                          {
                          if ( Cad_Longitud(Aux) > LONG_NOMBRE )
                            { Aux[LONG_NOMBRE] = NULO; }
                          Cad_Copia (Nombre, Aux);
                          Cad_Destruye (Aux);
                          TrataFichero (Directorio,Nombre,Extension);
                          }
                        break;

      case DIRECTORIO:  Aux = PideDirectorio(Directorio);
                        if ( Tec_Ultima() != TEC_ESC )
                          {
                          Cad_Copia (Directorio,Aux);
                          Refresca = SI;
                          }
                        Cad_Destruye (Aux);
                        break;

      case EXTENSN:     Aux = PideExtension(Extension);
                        if ( Tec_Ultima() != TEC_ESC )
                          {
                          Cad_Copia (Extension,Aux);
                          Refresca = SI;
                          }
                        Cad_Destruye (Aux);
                        break;

      case MASCARA:     Aux = PideMascara(Mascara);
                        if ( Tec_Ultima() != TEC_ESC )
                          {
                          Cad_Copia (Mascara,Aux);
                          Refresca = SI;
                          }
                        Cad_Destruye (Aux);
                        break;
      }
    LimpiaZonaPequena();
    }
  }

/*--------------------------------------------------------------------
 * FUNCION:  TrataFichero()
 * OBJETIVO: Preparar la conversin.
 * ENTRADAS: Directorio, nombre y extensin del fichero
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void TrataFichero (Dir,Nom,Ext)
cadena Dir,Nom,Ext;
  {
  cadena  NomPrf, NomRtf;
  fichero FichPrf, FichRtf;

  NomPrf = NombreCompleto(Dir,Nom,Ext);
  NomRtf = NombreCompleto(Dir,Nom,"rtf");

  if ( (FichPrf = Fch_AbreLeer (NomPrf,FCH_BINARIO)) )
    {
    if ( CompruebaPerfection (FichPrf) )
      {
      if ( (FichRtf = Fch_AbreGrabar (NomRtf,FCH_BINARIO)) )
        {
        LimpiaZonaPequena();
        Convierte (FichPrf, FichRtf);
        Fch_Cierra (FichRtf);
        }
      }
    Fch_Cierra (FichPrf);
    Usr_PulsaUnaTecla ("Conversin terminada");
    }

  Cad_Destruye (NomPrf);
  Cad_Destruye (NomRtf);
  }

/*--------------------------------------------------------------------
 * FUNCION:  Convierte()
 * OBJETIVO: Realizar la conversin
 * ENTRADAS: Los ficheros Perfection y RTF, abiertos y en posicin correcta
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void Convierte (Perfect, RichTex)
fichero Perfect;
fichero RichTex;
  {
  entero   i, j;
  octeto   Dato, TotalFuenteUsada, Estado=0;
  cadena   FuenteUsada[MAX_ESTILO], Aux;
  contador EstiloActual, Tamano;
  logico   HayQueCambiarEstilo, Visto;
  caracter InfoEstilo[MAX_ESTILO][50], Orden[50];

  Usr_Informa ("Convirtiendo");
  Cdr_Caja (CDR_SIMPLE,12,20,14,60,NEGRO,VERDE);
  Pan_Color (NEGRO,VERDE);
  Pan_PonTexto (13,23,"Octeto en conversin:");
  Pan_Tinta (BLANCO);
  Pan_CursorVisible (NO);

  TotalFuenteUsada = CalculaAsignacionFuente (FuenteUsada);
  Rtf_EscribeCadena (RichTex,"{\\rtf1\\pc\\deff0{\\fonttbl");
  Rtf_NuevaLinea (RichTex);
  for ( i = 0 ; i < TotalFuenteUsada ; i++ )
    {
    Visto = NO;
    for ( j=0 ; j<TotalFuente && !Visto ; j++ )
      {
      if ( Visto = Cad_Igual (FuenteUsada[i],FuenteDisponible[j]) )
        {
        switch ( TipoFuente[j] )
          {
          case EST_FUENTE_NIL:    Aux="nil";    break;
          case EST_FUENTE_ROMAN:  Aux="roman";  break;
          case EST_FUENTE_SWISS:  Aux="swiss";  break;
          case EST_FUENTE_MODERN: Aux="modern"; break;
          case EST_FUENTE_SCRIPT: Aux="script"; break;
          case EST_FUENTE_DECOR:  Aux="decor";  break;
          case EST_FUENTE_TECH:   Aux="tech";   break;
          }
        }
      }
    if ( !Visto )  { Aux = "nil"; }
    sprintf (Orden,"{\\f%d\\f%s %s;}",(contador)i,Aux,FuenteUsada[i]);
    Rtf_EscribeCadena (RichTex,Orden);
    if ( i != TotalFuenteUsada-1 )  { Rtf_NuevaLinea (RichTex); }
    }
  Rtf_EscribeCadena (RichTex,"}{\\stylesheet");
  Rtf_NuevaLinea (RichTex);
  for ( i = 0 ; i < MAX_ESTILO ; i++ )
    {
    HallaInfoEstilo (Estilo[i],InfoEstilo[i]);
    Rtf_EscribeCadena (RichTex,"{");
    Rtf_EscribeCadena (RichTex,InfoEstilo[i]);
    Rtf_EscribeEstilo (RichTex,Est_Nombre(Estilo[i]));
    Rtf_EscribeCadena (RichTex,";}");
    if ( i != MAX_ESTILO-1 )  { Rtf_NuevaLinea (RichTex); }
    }
  Rtf_EscribeCadena (RichTex,"}");
  Rtf_NuevaLinea (RichTex);

  i = 0;
  EstiloActual = 1;
  HayQueCambiarEstilo = SI;
  while ( Fch_LeeOcteto (Perfect,&Dato,1) )
    {
    i++;
    Pan_PonEntero (13,45,i,1);

    if ( HayQueCambiarEstilo )
      {
      if ( Dato != STRIP1 && Dato != STRIP2 && Dato != STRIP3 &&
           Dato != STRIP4 && Dato != STRIP5 && Dato != STRIP6 &&
           Dato != STRIP7 && Dato != STRIP8 )
        {
        sprintf (Orden,"\\s%d ",EstiloActual);
        Rtf_EscribeCadena (RichTex,Orden);
        Tamano = Est_Tamano(Estilo[EstiloActual-1]);
        HayQueCambiarEstilo = NO;
        }
      }

    if ( Dato>=32 && Dato<=127 )  { Fch_EscribeOcteto (RichTex,&Dato,1); }
    else
      {
      switch ( Dato )
        {
        case NUM_PAG_MAY:
        case NUM_PAG_MIN:
        case NUM_PAG_NUM:
        case NUM_PAG_ROM_MAY:
        case NUM_PAG_ROM_MIN:
        case PSEUDOESPACIO:
        case QUITAME:
        case RESERVADO:
        case INS_DIBUJO:
        case ERROR_ORTOGRAF:  break;

        case STRIP1: Rtf_EscribeCadena (RichTex,InfoEstilo[0]);
                     HayQueCambiarEstilo = NO;
                     Tamano = Est_Tamano(Estilo[0]);
                     EstiloActual = 1;
                     break;

        case STRIP2: Rtf_EscribeCadena (RichTex,InfoEstilo[1]);
                     HayQueCambiarEstilo = NO;
                     Tamano = Est_Tamano(Estilo[1]);
                     EstiloActual = 2;
                     break;

        case STRIP3: Rtf_EscribeCadena (RichTex,InfoEstilo[2]);
                     HayQueCambiarEstilo = NO;
                     Tamano = Est_Tamano(Estilo[2]);
                     EstiloActual = 3;
                     break;

        case STRIP4: Rtf_EscribeCadena (RichTex,InfoEstilo[3]);
                     HayQueCambiarEstilo = NO;
                     Tamano = Est_Tamano(Estilo[3]);
                     EstiloActual = 4;
                     break;

        case STRIP5: Rtf_EscribeCadena (RichTex,InfoEstilo[4]);
                     HayQueCambiarEstilo = NO;
                     Tamano = Est_Tamano(Estilo[4]);
                     EstiloActual = 5;
                     break;

        case STRIP6: Rtf_EscribeCadena (RichTex,InfoEstilo[5]);
                     HayQueCambiarEstilo = NO;
                     Tamano = Est_Tamano(Estilo[5]);
                     EstiloActual = 6;
                     break;

        case STRIP7: Rtf_EscribeCadena (RichTex,InfoEstilo[6]);
                     HayQueCambiarEstilo = NO;
                     Tamano = Est_Tamano(Estilo[6]);
                     EstiloActual = 7;
                     break;

        case STRIP8: Rtf_EscribeCadena (RichTex,InfoEstilo[7]);
                     HayQueCambiarEstilo = NO;
                     Tamano = Est_Tamano(Estilo[7]);
                     EstiloActual = 8;
                     break;

        case NEGRITA_SI:   Rtf_EscribeCadena (RichTex,"\\b1 ");
                           PonNegrita (Estado);
                           break;

        case NEGRITA_NO:   Rtf_EscribeCadena (RichTex,"\\b0 ");
                           QuitaNegrita (Estado);
                           break;

        case SUBRAYADO_SI: Rtf_EscribeCadena (RichTex,"\\scaps1 ");
                           PonVersalita (Estado);
                           break;

        case SUBRAYADO_NO: Rtf_EscribeCadena (RichTex,"\\scaps0 ");
                           QuitaVersalita (Estado);
                           break;

        case ITALICA_SI:     Rtf_EscribeCadena (RichTex,"\\i1 ");
                             PonItalica (Estado);
                             break;

        case SUPERINDICE_SI: PonSuperindice (Estado);
                             QuitaSubindice (Estado);
                             ExplicaEstado (Estado,Tamano,Orden);
                             Rtf_EscribeCadena (RichTex,Orden);
                             break;

        case SUBINDICE_SI:   PonSubindice (Estado);
                             QuitaSuperindice (Estado);
                             ExplicaEstado (Estado,Tamano,Orden);
                             Rtf_EscribeCadena (RichTex,Orden);
                             break;

        case ITALICA_SUB_SUP_NO:
                             QuitaItalica(Estado);
                             QuitaSubindice(Estado);
                             QuitaSuperindice(Estado);
                             ExplicaEstado (Estado,Tamano,Orden);
                             Rtf_EscribeCadena (RichTex,Orden);
                             break;

        case LETRA_NORMAL:   Rtf_EscribeCadena (RichTex,"\\plain ");
                             Estado = 0;
                             break;

        case FIN_LINEA:  Rtf_NuevaLinea (RichTex);
                         Rtf_EscribeCadena (RichTex," ");
                         break;

        case FIN_PARRAFO: Rtf_NuevaLinea (RichTex);
                          Rtf_EscribeCadena (RichTex,"\\par ");
                          HayQueCambiarEstilo = SI;
                          break;

        case FIN_PAGINA:  Rtf_NuevaLinea (RichTex);
                          Rtf_EscribeCadena (RichTex,"\\page ");
                          break;

        default: Rtf_EscribeCaracter (RichTex,Dato);
        }
      }
    }

  Rtf_EscribeCadena (RichTex,"}");
  }

/*--------------------------------------------------------------------
 * FUNCION:  CalculaAsignacionFuente()
 * OBJETIVO: Calcular el nmero de fuente que le corresponde a cada estilo
 * ENTRADAS: El vector de nombres de fuentes
 * SALIDAS:  El nmero total de fuentes usadas. La entrada se modifica
 *------------------------------------------------------------------*/
octeto CalculaAsignacionFuente (NombreFuente)
cadena NombreFuente[];
  {
  octeto Total=0, i, NumEstilo;
  logico Visto;

  for ( NumEstilo = 0 ; NumEstilo < MAX_ESTILO ; NumEstilo++ )
    {
    Visto = NO;
    for ( i = 0 ; i<Total && !Visto ; i++ )
      {
      if ( Visto = Cad_Igual (Est_Fuente(Estilo[NumEstilo]),NombreFuente[i]) )
        { Est_PonNumeroFuente(Estilo[NumEstilo],i); }
      }
    if ( !Visto )
      {
      Est_PonNumeroFuente(Estilo[NumEstilo],Total);
      NombreFuente[Total++] = Est_Fuente(Estilo[NumEstilo]);
      }
    }

  return ( Total );
  }

/*--------------------------------------------------------------------
 * FUNCION:  HallaInfoEstilo()
 * OBJETIVO: Escribir la cadena con toda la informacin del estilo
 * ENTRADAS: El estilo y la cadena donde dejar la informacin
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void HallaInfoEstilo (Estilo, Info)
estilo Estilo;
cadena Info;
  {
  char Justificacion;

  switch (Est_Justificacion(Estilo))
    {
    case EST_JUST_IZQUIERDA: Justificacion = 'l'; break;
    case EST_JUST_DERECHA:   Justificacion = 'r'; break;
    case EST_JUST_CENTRAL:   Justificacion = 'c'; break;
    case EST_JUST_COMPLETA:  Justificacion = 'j'; break;
    }

  sprintf (Info,"\\s%d\\li%d\\ri%d\\fi%d\\q%c\\f%d\\fs%d ",
      Est_Numero(Estilo),Twip(Est_MargenIzquierdo(Estilo)),
      Twip(Est_MargenDerecho(Estilo)),Twip(Est_MargenSangrado(Estilo)),
      Justificacion,Est_NumeroFuente(Estilo),2*Est_Tamano(Estilo));
  }

/*--------------------------------------------------------------------
 * FUNCION:  ExplicaEstado()
 * OBJETIVO: Escribir en una cadena el estado de resaltado de letra
 * ENTRADAS: El estado, la altura de la fuente y una cadena donde
 *           dejar la informacin. Tambin se usan dos variables globales
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void ExplicaEstado (Estado, Tamano, Cadena)
octeto   Estado;
contador Tamano;
cadena   Cadena;
  {
  caracter Aux[20];
  cadena   Pos;

  Tamano *= 2;

  if ( Subindice(Estado) )         { Pos = "\\dn"; }
  else if ( Superindice(Estado) )  { Pos = "\\up"; }

  if ( Subindice(Estado) || Superindice(Estado) )
    {
    sprintf (Aux, "%s%d\\fs%d ", Pos, PosicionSubSuper*Tamano/100,
                             TamanoSubSuper*Tamano/100);
    }
  else  { sprintf (Aux," "); }

  sprintf (Cadena, "\\plain\\b%d\\i%d\\scaps%d\\fs%d%s",
           Negrita(Estado),Italica(Estado),Versalita(Estado),Tamano,Aux);
  }

/*--------------------------------------------------------------------
 * FUNCION:  EligeUnNombre()
 * OBJETIVO: Mostrar al usuario una lista con los ficheros que puede leer
 *           y devolver el que elija
 * ENTRADAS: Directorio, mscara para el nombre, extensin del fichero y
 *           un nmero para retocar las dimensiones de la caja
 * SALIDAS:  Una cadena con el nombre (sin directorio ni extensin) o NIL
 * NOTA:     La cadena devuelta hay que destruirla cuando no sea necesaria
 *------------------------------------------------------------------*/
cadena EligeUnNombre (Dir,Nom,Ext,Num)
cadena Dir,Nom,Ext;
octeto Num;
  {
  contador Fil=10, Col=25, Ancho=26, Alto=11;
  cadena   Mascara, Mensaje, Respuesta = NIL;
  cadena   MenuPosible [MAX_FICHERO+1];
  lista    Posible, L;
  contador i, Elegido;
  logico   Sigue;

  Fil += Num;
  Alto -= Num;
  Num ? LimpiaZonaEstilo() : LimpiaZonaPequena();
  Cdr_Caja (CDR_SIMPLE,Fil,Col,Fil+Alto,Col+Ancho,NEGRO,VERDE);

  Mascara = NombreCompleto(Dir,Nom,Ext);
  Usr_Informa ("Buscando ficheros");
  Posible = Fch_ListaNombre (Mascara);
  if ( Posible )
    {
    for ( i=0 , L=Posible ; L && i<MAX_FICHERO ; L=Lis_Siguiente(L) )
      { MenuPosible[i++] = Lis_Contenido(L); }
    MenuPosible[i] = NIL;

    Pan_Color (NEGRO,VERDE);
    Pan_PonTexto (Fil+1,Col+4,"Elije el fichero:");
    if ( Elegido =
         Men_Vertical (Fil+3,Col+4,Fil+Alto-1,Col+20,MenuPosible,(entero)1) )
      {
      Sigue = SI;
      for ( i=Cad_Longitud(MenuPosible[Elegido-1]) ; i>=0 && Sigue ; i-- )
        {
        if ( MenuPosible[Elegido-1][i] == Fch_Separador() )
          {
          MenuPosible[Elegido-1][i] = NULO;
          Sigue = NO;
          }
        }
      Respuesta = Cad_Duplica (MenuPosible[Elegido-1]);
      }
    Lis_Destruye (Posible);
    }

  else
    {
    Mensaje = Cad_Une ("No se encontrado ningn fichero ","\"",Mascara,
                       "\"", NIL);
    Usr_PulsaUnaTecla (Mensaje);
    Cad_Destruye (Mensaje);
    }

  Cad_Destruye (Mascara);
  return ( Respuesta );
  }

/*--------------------------------------------------------------------
 * FUNCION:  NombreCompleto()
 * OBJETIVO: Devolver el nombre completo de un fichero
 * ENTRADAS: Directorio, nombre y extensin del fichero
 * SALIDAS:  Una cadena con el nombre completo
 * NOTA:     La cadena hay que destruirla cuando no sea necesaria
 *------------------------------------------------------------------*/
cadena NombreCompleto (Dir,Nom,Ext)
cadena Dir,Nom,Ext;
  {
  cadena Respuesta, Aux;

  Aux = Fch_Nombre (Nom,Ext);
  Respuesta = Cad_Une (Dir,Aux,NIL);
  Cad_Destruye (Aux);

  return ( Respuesta );
  }

/*--------------------------------------------------------------------
 * FUNCION:  PideDirectorio()
 * OBJETIVO: Pedir al usuario un directorio
 * ENTRADAS: El directorio propuesto
 * SALIDAS:  El nuevo directorio
 * NOTA:     La cadena hay que destruirla cuando no sea necesaria
 *------------------------------------------------------------------*/
cadena PideDirectorio (Dir)
cadena Dir;
  {
  static contador Fil=13, Col=22, Ancho=36, Alto=6;

  Cdr_Caja (CDR_SIMPLE,Fil,Col,Fil+Alto,Col+Ancho,NEGRO,VERDE);

  Pan_Color (NEGRO,VERDE);
  Pan_PonTexto (Fil+2,Col+6,"Escribe el directorio:");
  return ( Usr_Texto (Dir,LONG_DIRECTORIO,Fil+4,Col+6,NEGRO,BLANCO) );
  }

/*--------------------------------------------------------------------
 * FUNCION:  PideExtension()
 * OBJETIVO: Pedir al usuario una extensin
 * ENTRADAS: La extensin propuesta
 * SALIDAS:  La nueva extensin
 * NOTA:     La cadena hay que destruirla cuando no sea necesaria
 *------------------------------------------------------------------*/
cadena PideExtension (Ext)
cadena Ext;
  {
  static contador Fil=13, Col=23, Ancho=34, Alto=6;

  Cdr_Caja (CDR_SIMPLE,Fil,Col,Fil+Alto,Col+Ancho,NEGRO,VERDE);

  Pan_Color (NEGRO,VERDE);
  Pan_PonTexto (Fil+2,Col+6,"Escribe la extensin:");
  return ( Usr_Texto (Ext,LONG_EXTENSION,Fil+4,Col+8,NEGRO,BLANCO) );
  }

/*--------------------------------------------------------------------
 * FUNCION:  PideMascara()
 * OBJETIVO: Pedir al usuario una mscara
 * ENTRADAS: La mscara propuesta
 * SALIDAS:  La nueva mscara
 * NOTA:     La cadena hay que destruirla cuando no sea necesaria
 *------------------------------------------------------------------*/
cadena PideMascara (Mas)
cadena Mas;
  {
  static contador Fil=13, Col=24, Ancho=32, Alto=6;

  Cdr_Caja (CDR_SIMPLE,Fil,Col,Fil+Alto,Col+Ancho,NEGRO,VERDE);

  Pan_Color (NEGRO,VERDE);
  Pan_PonTexto (Fil+2,Col+6,"Escribe la mscara:");
  return ( Usr_Texto (Mas,LONG_MASCARA,Fil+4,Col+8,NEGRO,BLANCO) );
  }

/*--------------------------------------------------------------------
 * FUNCION:  PideNombre()
 * OBJETIVO: Pedir al usuario un nombre de fichero
 * ENTRADAS: El nombre propuesto y la mxima longitud admitida
 * SALIDAS:  El nuevo nombre
 * NOTA:     La cadena hay que destruirla cuando no sea necesaria
 *------------------------------------------------------------------*/
cadena PideNombre (Nom,MaxLong)
cadena   Nom;
contador MaxLong;
  {
  static contador Fil=13, Col=22, Ancho=36, Alto=6;

  Cdr_Caja (CDR_SIMPLE,Fil,Col,Fil+Alto,Col+Ancho,NEGRO,VERDE);

  Pan_Color (NEGRO,VERDE);
  Pan_PonTexto (Fil+2,Col+4,"Escribe el nombre del fichero:");
  return ( Usr_Texto (Nom,MaxLong,Fil+4,Col+6,NEGRO,BLANCO) );
  }

/*--------------------------------------------------------------------
 * FUNCION:  GestionaEstilo()
 * OBJETIVO: Dirigir la gestin de estilos
 * ENTRADAS: Ninguna
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void GestionaEstilo()
  {
  enum { SALIDA_ESC, LEE, DEFINE, GRABA, DIRECTORIO, MASCARA, EXTENSN };

  static cadena MenuEstilo[] = {">Lee", "De>fine", ">Graba", ">Directorio",
                                ">Mscara", ">Extensin", NIL };

  static char Directorio [LONG_DIRECTORIO+1];
  static char Nombre [LONG_NOMBRE+1];
  static char Mascara [LONG_MASCARA+1] = "*";
  static char Extension [LONG_EXTENSION+1] = "est";

  cadena   Aux;
  entero   Opcion;
  contador Fil=4, Col=20, Ancho=40, Alto=7;
  logico   Sigue = SI, Refresca = SI;

  Opcion = DEFINE;
  while ( Sigue )
    {
    if ( Refresca )
      {
      Cdr_Caja (CDR_SIMPLE,Fil,Col,Fil+Alto,Col+Ancho,NEGRO,VERDE);
      Pan_Borra (NEGRO,Fil+2,Col+15,Fil+Alto-1,Col+Ancho-1);
      Pan_Color (NEGRO,VERDE);
      Pan_PonTexto (Fil+4,Col+15,Directorio);
      Pan_PonTexto (Fil+5,Col+15,Mascara);
      Pan_PonTexto (Fil+6,Col+15,Extension);
      Refresca = NO;
      }

    Opcion = Men_Vertical (Fil+1,Col+1,Fil+Alto-1,Col+13,MenuEstilo,Opcion);
    switch ( Opcion )
      {
      case SALIDA_ESC:  Sigue = NO;      break;

      case LEE:         Aux = EligeUnNombre (Directorio, Mascara,
                                             Extension,2);
                        if ( Aux )
                          {
                          if ( Cad_Longitud(Aux) > LONG_NOMBRE )
                            { Aux[LONG_NOMBRE] = NULO; }
                          Cad_Copia (Nombre, Aux);
                          Cad_Destruye (Aux);

                          Aux = NombreCompleto (Directorio,Nombre,Extension);
                          LeeFicheroEstilo (Aux);
                          Usr_PulsaUnaTecla ("Fichero leido");
                          Cad_Destruye (Aux);
                          }
                        break;

      case DEFINE:      DefineTablaEstilo();
                        Refresca = SI;
                        break;

      case GRABA:       Aux = PideNombre (Nombre,LONG_NOMBRE);
                        if ( Tec_Ultima() != TEC_ESC )
                          {
                          Cad_Copia (Nombre,Aux);
                          Cad_Destruye (Aux);
                          Aux = NombreCompleto (Directorio,Nombre,Extension);
                          GrabaFicheroEstilo (Aux);
                          }
                        Cad_Destruye (Aux);
                        break;

      case DIRECTORIO:  Aux = PideDirectorio(Directorio);
                        if ( Tec_Ultima() != TEC_ESC )
                          {
                          Cad_Copia (Directorio,Aux);
                          Refresca = SI;
                          }
                        Cad_Destruye (Aux);
                        break;

      case EXTENSN:     Aux = PideExtension(Extension);
                        if ( Tec_Ultima() != TEC_ESC )
                          {
                          Cad_Copia (Extension,Aux);
                          Refresca = SI;
                          }
                        Cad_Destruye (Aux);
                        break;

      case MASCARA:     Aux = PideMascara(Mascara);
                        if ( Tec_Ultima() != TEC_ESC )
                          {
                          Cad_Copia (Mascara,Aux);
                          Refresca = SI;
                          }
                        Cad_Destruye (Aux);
                        break;
      }
    LimpiaZonaEstilo();
    }
  }

/*--------------------------------------------------------------------
 * FUNCION:  GrabaFicheroEstilo()
 * OBJETIVO: Grabar los estilos en un fichero
 * ENTRADAS: El nombre del fichero y la variable global Estilo
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void GrabaFicheroEstilo (Nombre)
cadena Nombre;
  {
  fichero  Fichero;
  cadena   Aux;
  caracter Linea[80];
  contador i;

  if ( Fichero = Fch_AbreGrabar (Nombre, FCH_TEXTO) )
    {
    Fch_EscribeLinea (Fichero, "* Fichero de estilos para PerfRTF");
    for ( i=0 ; i<MAX_ESTILO ; i++ )
      {
      Fch_EscribeLinea (Fichero, "");
      sprintf ( Linea, "Estilo %d", i+1);
      Fch_EscribeLinea (Fichero, Linea);
      sprintf ( Linea, "Nombre %s", Est_Nombre(Estilo[i]) );
      Fch_EscribeLinea (Fichero, Linea);
      sprintf ( Linea, "Fuente %s", Est_Fuente(Estilo[i]) );
      Fch_EscribeLinea (Fichero, Linea);
      sprintf ( Linea, "Tamao %d", Est_Tamano(Estilo[i]) );
      Fch_EscribeLinea (Fichero, Linea);
      switch ( Est_Justificacion(Estilo[i]) )
        {
        case EST_JUST_IZQUIERDA: Aux = "Izquierda"; break;
        case EST_JUST_DERECHA:   Aux = "Derecha";   break;
        case EST_JUST_CENTRAL:   Aux = "Central";   break;
        case EST_JUST_COMPLETA:  Aux = "Completa";  break;
        }
      sprintf ( Linea, "Justificacin   %s", Aux );
      Fch_EscribeLinea (Fichero, Linea);
      sprintf ( Linea, "MargenIzquierdo %d", Est_MargenIzquierdo(Estilo[i]) );
      Fch_EscribeLinea (Fichero, Linea);
      sprintf ( Linea, "MargenDerecho   %d", Est_MargenDerecho(Estilo[i]) );
      Fch_EscribeLinea (Fichero, Linea);
      sprintf ( Linea, "MargenSangrado  %d", Est_MargenSangrado(Estilo[i]) );
      Fch_EscribeLinea (Fichero, Linea);
      }
    Fch_Cierra (Fichero);
    Usr_PulsaUnaTecla ("Fichero escrito");
    }
  }

/*--------------------------------------------------------------------
 * FUNCION:  DefineTablaEstilo()
 * OBJETIVO: Definir los ocho estilos del programa
 * ENTRADAS: Ninguna
 * SALIDAS:  Cambian los estilos
 *------------------------------------------------------------------*/
void DefineTablaEstilo()
  {
  static cadena MenuTabla[] = {">Uno", ">Dos", ">Tres", ">Cuatro", "C>inco",
                               ">Seis", "Si>ete", ">Ocho", ">Bien", NIL };
  static contador Fil=6, Col=2, Ancho=26, Alto=10;

  estilo   Provisional [MAX_ESTILO];
  cadena   Aux;
  entero   Opcion;
  contador i;
  logico   Sigue = SI, Cambia;

  Pan_Borra (NEGRO,4,0,21,79);
  Cdr_Caja (CDR_SIMPLE,4,0,21,79,NEGRO,VERDE);
  Pan_Color (NEGRO,ROJO);
  Pan_PonTexto (5,28,"Definicin de los estilos");

  Cdr_Caja (CDR_SIMPLE,Fil,Col,Fil+Alto,Col+Ancho,NEGRO,VERDE);

  for ( i=0 ; i<MAX_ESTILO ; i++ )
    { Provisional[i] = Est_Duplica(Estilo[i]); }

  Opcion = 1;
  while ( Sigue )
    {
    Pan_Borra (NEGRO,Fil+1,Col+8,Fil+Alto-1,Col+Ancho-1);
    Pan_Color (NEGRO,VERDE);
    for ( i=0 ; i<MAX_ESTILO ; i++ )
      { Pan_PonTexto (Fil+1+i,Col+10,Est_Nombre(Provisional[i])); }

    Opcion = Men_Vertical (Fil+1,Col+1,Fil+Alto-1,Col+9,MenuTabla,Opcion);
    if ( !Opcion )
      {
      Sigue = NO;
      Cambia = NO;
      }
    else if ( Opcion == 9 )
      {
      Sigue = NO;
      Cambia = SI;
      }
    else
      { Provisional[Opcion-1] = DefineEstilo(Provisional[Opcion-1]); }
    }
  Pan_Borra (NEGRO,4,0,21,79);

  if ( Cambia )
    {
    for ( i=0 ; i<MAX_ESTILO ; i++ )
      {
      Est_Destruye (Estilo[i]);
      Estilo[i] = Est_Duplica (Provisional[i]);
      }
    }

  for ( i=0 ; i<MAX_ESTILO ; i++ )
    { Est_Destruye(Provisional[i]); }
  }

/*--------------------------------------------------------------------
 * FUNCION:  DefineEstilo()
 * OBJETIVO: Definir un estilo
 * ENTRADAS: Un estilo
 * SALIDAS:  El estilo con los cambios, si los hay
 *------------------------------------------------------------------*/
estilo DefineEstilo (Estilo)
estilo Estilo;
  {
  enum { SALIDA_ESC, NOMBRE, FUENTE, TAMANO, JUSTIFICA, MARGENIZQ, MARGENDER,
         MARGENSAN, BIEN };

  static cadena Caracteristica[] = {">Nombre", ">Fuente", ">Tamao",
                ">Justificacin", "M. >Izquierdo", "M. >Derecho",
                "M. >Sangrado", ">Bien", NIL };
  static contador Fil=6, Col=31, Ancho=33, Alto=9;

  cadena   Aux;
  estilo   Nuevo, Respuesta;
  entero   Opcion;
  contador i, Num;
  logico   Sigue = SI, Bien;

  Cdr_Caja (CDR_DOBLE,Fil,Col,Fil+Alto,Col+Ancho,NEGRO,VERDE);

  Nuevo = Est_Duplica (Estilo);

  Opcion = NOMBRE;
  while ( Sigue )
    {
    Pan_Borra (NEGRO,Fil+1,Col+17,Fil+Alto-1,Col+Ancho-1);
    Pan_Color (NEGRO,VERDE);
    Pan_PonTexto  (Fil+1,Col+17,Est_Nombre(Nuevo));
    Pan_PonTexto  (Fil+2,Col+17,Est_Fuente(Nuevo));
    Pan_PonEntero (Fil+3,Col+17,Est_Tamano(Nuevo),1);
    Pan_Cursor (Fil+4,Col+17);
    switch (Est_Justificacion(Nuevo))
      {
      case EST_JUST_IZQUIERDA: Pan_Texto ("Izquierda"); break;
      case EST_JUST_DERECHA:   Pan_Texto ("Derecha");   break;
      case EST_JUST_CENTRAL:   Pan_Texto ("Central");   break;
      case EST_JUST_COMPLETA:  Pan_Texto ("Completa");  break;
      }
    Pan_PonEntero (Fil+5,Col+17,Est_MargenIzquierdo(Nuevo),1);
    Pan_PonEntero (Fil+6,Col+17,Est_MargenDerecho(Nuevo),1);
    Pan_PonEntero (Fil+7,Col+17,Est_MargenSangrado(Nuevo),1);

    Opcion = Men_Vertical (Fil+1,Col+1,Fil+Alto-1,Col+16,
                           Caracteristica,Opcion);
    switch ( Opcion )
      {
      case SALIDA_ESC: Sigue = NO; Bien = NO; break;
      case NOMBRE:     Aux = PideNombreEstilo (Est_Nombre(Nuevo),
                                               LONG_NOMBRE_EST);
                        if ( Tec_Ultima() != TEC_ESC )
                          { Est_PonNombre (Nuevo,Aux); }
                        Cad_Destruye (Aux);
                        break;
      case FUENTE:      if ( Num = PideFuenteEstilo() )
                          { Est_PonFuente (Nuevo,FuenteDisponible[Num-1]); }
                        break;
      case TAMANO:     Est_PonTamano (Nuevo,
                           PideTamanoEstilo(Est_Tamano(Nuevo)));
                       break;
      case JUSTIFICA:  Est_PonJustificacion (Nuevo,
                           PideJustificacionEstilo(Est_Justificacion(Nuevo)));
                       break;
      case MARGENIZQ:  Est_PonMargenIzquierdo (Nuevo,
                           PideMargenEstilo(Est_MargenIzquierdo(Nuevo),0,250));
                       break;
      case MARGENDER:  Est_PonMargenDerecho (Nuevo,
                           PideMargenEstilo(Est_MargenDerecho(Nuevo),0,250));
                       break;
      case MARGENSAN:  Est_PonMargenSangrado (Nuevo,
                           PideMargenEstilo(Est_MargenSangrado(Nuevo),
                                            -250,250));
                       break;
      case BIEN:       Sigue = NO; Bien = SI; break;
      }
    }

  Pan_Borra (NEGRO,Fil,Col,Fil+Alto,Col+Ancho);

  if ( Bien )
    {
    Respuesta = Nuevo;
    Est_Destruye (Estilo);
    }
  else
    {
    Respuesta = Estilo;
    Est_Destruye (Nuevo);
    }

  return ( Respuesta );
  }

/*--------------------------------------------------------------------
 * FUNCION:  PideNombreEstilo()
 * OBJETIVO: Pedir al usuario un nombre de estilo
 * ENTRADAS: El nombre propuesto y la mxima longitud admitida
 * SALIDAS:  El nuevo nombre
 * NOTA:     La cadena hay que destruirla cuando no sea necesaria
 *------------------------------------------------------------------*/
cadena PideNombreEstilo (Nom,MaxLong)
cadena   Nom;
contador MaxLong;
  {
  cadena Respuesta;
  static contador Fil=18, Col=15, Ancho=50, Alto=2;

  Cdr_Caja (CDR_DOBLE,Fil,Col,Fil+Alto,Col+Ancho,NEGRO,BLANCO);

  Pan_Color (NEGRO,VERDE);
  Pan_PonTexto (Fil+1,Col+2,"Escribe el nombre del estilo:");
  Respuesta = Usr_Texto (Nom,MaxLong,Fil+1,Col+32,NEGRO,BLANCO);
  Pan_Borra (NEGRO,Fil,Col,Fil+Alto,Col+Ancho);

  return ( Respuesta );
  }

/*--------------------------------------------------------------------
 * FUNCION:  PideFuenteEstilo()
 * OBJETIVO: Pedir al usuario una fuente
 * ENTRADAS: Ninguna
 * SALIDAS:  El nmero de la fuente elegido o 0
 *------------------------------------------------------------------*/
contador PideFuenteEstilo()
  {
  static contador Fil=16, Col=35, Ancho=25, Alto=4;
  contador Respuesta;

  Pan_Color (NEGRO,VERDE);
  Pan_PonTexto (Fil+2,Col-18,"Elige la fuente:");
  Respuesta = Men_Vertical (Fil,Col,Fil+Alto,Col+Ancho,FuenteDisponible,
                            (entero)1);
  Pan_Borra (NEGRO,Fil,Col,Fil+Alto,Col+Ancho);
  Pan_Borra (NEGRO,Fil+2,Col-20,Fil+2,Col);

  return ( Respuesta );
  }

/*--------------------------------------------------------------------
 * FUNCION:  PideTamanoEstilo()
 * OBJETIVO: Pedir al usuario un tamao
 * ENTRADAS: El tamao propuesto y los valores mnimo y mximo admitidos
 * SALIDAS:  El nuevo tamao
 *------------------------------------------------------------------*/
octeto PideTamanoEstilo (Tamano)
octeto Tamano;
  {
  octeto Respuesta;
  static contador Fil=18, Col=25, Ancho=30, Alto=2;

  Cdr_Caja (CDR_DOBLE,Fil,Col,Fil+Alto,Col+Ancho,NEGRO,BLANCO);

  Pan_Color (NEGRO,VERDE);
  Pan_PonTexto (Fil+1,Col+2,"Elige el tamao:");
  Respuesta = Usr_Entero (Tamano, 4, 0, 250, Fil+1, Col+19, NEGRO, BLANCO);
  Pan_Borra (NEGRO,Fil,Col,Fil+Alto,Col+Ancho);

  return ( Respuesta );
  }

/*--------------------------------------------------------------------
 * FUNCION:  PideJustificacionEstilo()
 * OBJETIVO: Pedir al usuario la justificacin del estilo
 * ENTRADAS: La justificacin propuesta
 * SALIDAS:  La nueva
 *------------------------------------------------------------------*/
octeto PideJustificacionEstilo (Justif)
octeto Justif;
  {
  static cadena MenuJustif[] = {">Izquierda", ">Derecha", ">Central",
                                "Com>pleta", NIL };
  octeto Respuesta;
  static contador Fil=17, Col=15, Ancho=50, Alto=3;

  Cdr_Caja (CDR_DOBLE,Fil,Col,Fil+Alto,Col+Ancho,NEGRO,BLANCO);

  Pan_Color (NEGRO,VERDE);
  Pan_PonTexto (Fil+1,Col+12,"Elige la justificacin:");
  Respuesta = Men_Horizontal (Fil+2,Col+2,Col+Ancho-1,MenuJustif,
                              (entero)Justif+1);
  if ( !Respuesta )  { Respuesta = Justif; }
  else               { Respuesta--; }
  Pan_Borra (NEGRO,Fil,Col,Fil+Alto,Col+Ancho);

  return ( Respuesta );
  }

/*--------------------------------------------------------------------
 * FUNCION:  PideMargenEstilo()
 * OBJETIVO: Pedir al usuario un margen
 * ENTRADAS: El margen propuesto y los valores mnimo y mximo admitidos
 * SALIDAS:  El nuevo margen
 *------------------------------------------------------------------*/
contador PideMargenEstilo (Margen,Min,Max)
contador Margen, Min, Max;
  {
  contador Respuesta;
  static contador Fil=18, Col=25, Ancho=30, Alto=2;

  Cdr_Caja (CDR_DOBLE,Fil,Col,Fil+Alto,Col+Ancho,NEGRO,BLANCO);

  Pan_Color (NEGRO,VERDE);
  Pan_PonTexto (Fil+1,Col+2,"Elige el margen:");
  Respuesta = Usr_Entero (Margen, 4, Min, Max, Fil+1, Col+19, NEGRO, BLANCO);
  Pan_Borra (NEGRO,Fil,Col,Fil+Alto,Col+Ancho);

  return ( Respuesta );
  }

/*--------------------------------------------------------------------
 * FUNCION:  LeeConfig()
 * OBJETIVO: Leer un fichero de configuracin
 *------------------------------------------------------------------*/
void LeeConfig (Nombre)
cadena Nombre;
  {
  config    Config;
  cadena    Linea;
  char     *Etiqueta, *Aux, *Aux2;
  logico    CabeFuente = SI;

  Usr_Informa ("Leyendo configuracin");

  if ( Config = Cnf_Abre (Nombre) )
    {
    while ( Linea = Cnf_Lee (Config) )
      {
      Cad_RecortaFinal (Linea);
      if ( Etiqueta = strtok (Linea, " ") )
        {

        if ( Cad_Igual (Etiqueta,"Sonido") )
          {
          if ( Aux = strtok (NULL, " ") )
            {
            if ( toupper(Aux[0]) == 'S' ) { Son_Enciende(); }
            else                          { Son_Apaga(); }
            }
          }

        else if ( Cad_Igual (Etiqueta,"Extensin") )
          {
          if ( Aux = strtok (NULL, " ") )
            {
            if ( Cad_Longitud(Aux) > 3 )  { Aux[3] = NULO; }
            Cad_Copia (ExtensionPerfection,Aux);
            }
          }

        else if ( Cad_Igual (Etiqueta,"FicheroEstilo") )
          {
          if ( Aux = strtok (NULL, " ") )
            { FicheroEstilo = Fch_Nombre (Aux,"est"); }
          }

        else if ( Cad_Igual (Etiqueta,"TamaoSubSuper") )
          {
          if ( Aux = strtok (NULL, " ") )
            {
            TamanoSubSuper = atoi (Aux);
            TamanoSubSuper = Min (TamanoSubSuper,100);
            }
          }

        else if ( Cad_Igual (Etiqueta,"PosicinSubSuper") )
          {
          if ( Aux = strtok (NULL, " ") )
            {
            PosicionSubSuper = atoi (Aux);
            PosicionSubSuper = Min (PosicionSubSuper,100);
            }
          }

        else if ( Cad_Igual (Etiqueta,"Fuente") )
          {
          if ( CabeFuente &&
               (Aux=strtok(NULL, " ")) &&
               (Aux2=strtok(NULL, "\0")) )
            {
            CabeFuente = AlmacenaFuente (Aux,Aux2+Cad_PrimerUtil(Aux2)-1);
            }
          }

        } /* Fin if Etiqueta */
      Cad_Destruye (Linea);
      } /* Fin while Linea */
    } /* Fin if Cnf_Abre() */

  if ( !CabeFuente )
    { Usr_Avisa ("No se han podido almacenar todas las fuentes"); }
  }

/*--------------------------------------------------------------------
 * FUNCION:  LeeFicheroEstilo()
 * OBJETIVO: Leer un fichero con definiciones de estilos
 *------------------------------------------------------------------*/
void LeeFicheroEstilo (Nombre)
cadena Nombre;
  {
  config    Config;
  cadena    Linea;
  contador  NumEstilo;
  char     *Etiqueta, *Aux, *Aux2;

  Usr_Informa ("Leyendo fichero de estilos");

  if ( Config = Cnf_Abre (Nombre) )
    {
    while ( Linea = Cnf_Lee (Config) )
      {
      Cad_RecortaFinal (Linea);
      if ( Etiqueta = strtok (Linea, " ") )
        {

        if ( Cad_Igual (Etiqueta,"Estilo") )
          {
          if ( Aux = strtok (NULL, " ") )
            {
            NumEstilo = atoi (Aux);
            if ( NumEstilo<1 || NumEstilo>8 )
              {
              Usr_ErrorFatal (
                  "Error fatal: hay un nmero de estilo no permitido");
              }
            else
              {
              NumEstilo--;
              Est_PonNumero (Estilo[NumEstilo],NumEstilo+1);
              }
            }
          }

        else if ( Cad_Igual (Etiqueta,"Nombre") )
          {
          if ( Aux = strtok (NULL, "\0") )
            { Est_PonNombre (Estilo[NumEstilo],Aux+Cad_PrimerUtil(Aux)-1); }
          }

        else if ( Cad_Igual (Etiqueta,"Fuente") )
          {
          if ( Aux = strtok (NULL, "\0") )
            { Est_PonFuente (Estilo[NumEstilo],Aux+Cad_PrimerUtil(Aux)-1); }
          }

        else if ( Cad_Igual (Etiqueta,"Tamao") )
          {
          if ( Aux = strtok (NULL, " ") )
            { Est_PonTamano (Estilo[NumEstilo],atoi(Aux)); }
          }

        else if ( Cad_Igual (Etiqueta,"Justificacin") )
          {
          if ( Aux = strtok (NULL, " ") )
            {
            if ( Cad_Igual(Aux,"Izquierda") )
              { Est_PonJustificacion (Estilo[NumEstilo],EST_JUST_IZQUIERDA); }
            if ( Cad_Igual(Aux,"Derecha") )
              { Est_PonJustificacion (Estilo[NumEstilo],EST_JUST_DERECHA); }
            if ( Cad_Igual(Aux,"Central") )
              { Est_PonJustificacion (Estilo[NumEstilo],EST_JUST_CENTRAL); }
            if ( Cad_Igual(Aux,"Completa") )
              { Est_PonJustificacion (Estilo[NumEstilo],EST_JUST_COMPLETA); }
            }
          }

        else if ( Cad_Igual (Etiqueta,"MargenIzquierdo") )
          {
          if ( Aux = strtok (NULL, " ") )
            { Est_PonMargenIzquierdo (Estilo[NumEstilo],atoi(Aux)); }
          }

        else if ( Cad_Igual (Etiqueta,"MargenDerecho") )
          {
          if ( Aux = strtok (NULL, " ") )
            { Est_PonMargenDerecho (Estilo[NumEstilo],atoi(Aux)); }
          }

        else if ( Cad_Igual (Etiqueta,"MargenSangrado") )
          {
          if ( Aux = strtok (NULL, " ") )
            { Est_PonMargenSangrado (Estilo[NumEstilo],atoi(Aux)); }
          }

        } /* Fin if Etiqueta */
      Cad_Destruye (Linea);
      } /* Fin while Linea */
    } /* Fin if Cnf_Abre() */
  }

/*--------------------------------------------------------------------
 * FUNCION:  AlmacenaFuente()
 * OBJETIVO: Almacenar el tipo y el nombre de una fuente
 * ENTRADAS: El tipo y el nombre
 * SALIDAS:  Lgica, que indica si se ha podido almacenar.
 *           Las variables globales pueden cambiar
 *------------------------------------------------------------------*/
logico AlmacenaFuente (Tipo, Nombre)
cadena Tipo, Nombre;
  {
  logico Respuesta, VistoTipo=NO;

  if ( Cad_Igual(Tipo,"nil") )
      { TipoFuente[TotalFuente] = EST_FUENTE_NIL; VistoTipo = SI; }
  if ( Cad_Igual(Tipo,"roman") )
      { TipoFuente[TotalFuente] = EST_FUENTE_ROMAN; VistoTipo = SI; }
  if ( Cad_Igual(Tipo,"swiss") )
      { TipoFuente[TotalFuente] = EST_FUENTE_SWISS; VistoTipo = SI; }
  if ( Cad_Igual(Tipo,"modern") )
      { TipoFuente[TotalFuente] = EST_FUENTE_MODERN; VistoTipo = SI; }
  if ( Cad_Igual(Tipo,"script") )
      { TipoFuente[TotalFuente] = EST_FUENTE_SCRIPT; VistoTipo = SI; }
  if ( Cad_Igual(Tipo,"decor") )
      { TipoFuente[TotalFuente] = EST_FUENTE_DECOR; VistoTipo = SI; }
  if ( Cad_Igual(Tipo,"tech") )
      { TipoFuente[TotalFuente] = EST_FUENTE_TECH; VistoTipo = SI; }
  if ( !VistoTipo )
      { TipoFuente[TotalFuente] = EST_FUENTE_NIL; }

  FuenteDisponible[TotalFuente] = Cad_Duplica(Nombre);

  if ( TotalFuente < MAX_FUENTE-1 )
    {
    TotalFuente++;
    Respuesta = SI;
    }
  else  { Respuesta = NO; }

  return ( Respuesta );
  }

/*--------------------------------------------------------------------
 * FUNCION:  CompruebaPerfection()
 * OBJETIVO: Comprobar que un fichero tiene formato Perfection SE y
 *           dejarlo en la posicin adecuada para tratarlo
 * ENTRADAS: El fichero Perfection, abierto y en posicin 0
 * SALIDAS:  Lgica
 * ALGORITMO:
 *      Los 4 primeros octetos son la signatura del fichero
 *      Los dos siguientes bloques, idnticos, son la cabecera y el
 *                pie de pgina
 *      El ltimo bloque es el de tabuladores y longitud de pgina
 *------------------------------------------------------------------*/
logico CompruebaPerfection (Fichero)
fichero Fichero;
  {
  logico   Error=NO;
  octeto   Dato[4];
  entero   Avance, i;
  contador Vez;

  if ( !Fch_LeeOcteto (Fichero,Dato,4) )  { Error = SI; }

  if ( !Error )
    {
    if ( (Dato[0] != 255) || (Dato[1] != 1) ||
         (Dato[2] != 67)  || (Dato[3] != 88) )
      { Error = SI; }
    }

  if ( !Error )
    {
    for ( Vez = 0 ; Vez<2 && !Error ; Vez++ )
      {
      if ( !Fch_LeeOcteto (Fichero,Dato,2) )  { Error = SI; }
      Avance = 256 * Dato[0] + Dato[1] + 4;
      for ( i = 0 ; !Error && i<Avance ; i++ )
        { if ( !Fch_LeeOcteto (Fichero,Dato,1) )  { Error = SI; } }
      }
    }

  if ( !Error )
    {
    for ( i = 0 ; !Error && i<258 ; i++ )
    if ( !Fch_LeeOcteto (Fichero,Dato,1) )  { Error = SI; }
    }

  if ( Error )
    { Usr_Avisa ("El fichero no tiene formato correcto"); }

  return ( !Error );
  }

/*--------------------------------------------------------------------
 * FUNCION:  Rtf_EscribeCadena()
 * OBJETIVO: Escribir en un fichero RTF una cadena
 * ENTRADAS: El fichero y la cadena
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void Rtf_EscribeCadena (Fichero, Cadena)
fichero Fichero;
cadena  Cadena;
  {
  Fch_EscribeOcteto (Fichero,Cadena,Cad_Longitud(Cadena));
  }

/*--------------------------------------------------------------------
 * FUNCION:  Rtf_EscribeCaracter()
 * OBJETIVO: Escribir en un fichero RTF un octeto
 * ENTRADAS: El fichero y el octeto
 * SALIDAS:  Ninguna
 * NOTA:     Esta funcin es especfica para este programa, no es general
 *------------------------------------------------------------------*/
void Rtf_EscribeCaracter (Fichero,Octeto)
fichero Fichero;
octeto  Octeto;
  {
  if ( Octeto < 32 )  { Fch_EscribeOcteto (Fichero,&Octeto,1); }
  else
    {
    Octeto = Car_ConvierteQLaPC ((caracter)Octeto);
    CarExt[2] = Digito [Octeto / 16];
    CarExt[3] = Digito [Octeto % 16];
    Fch_EscribeOcteto (Fichero,CarExt,4);
    }
  }

/*--------------------------------------------------------------------
 * FUNCION:  Rtf_EscribeEstilo()
 * OBJETIVO: Escribir en un fichero RTF el nombre de un estilo
 * ENTRADAS: El fichero y la cadena con el nombre
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void Rtf_EscribeEstilo (Fichero,Nombre)
fichero Fichero;
cadena  Nombre;
  {
  contador i;
  octeto   Octeto;

  for ( i=0 ; Octeto = (octeto)Nombre[i] ; i++ )
    {
    if ( Octeto < 127 )  { Fch_EscribeOcteto (Fichero,&Octeto,1); }
    else
      {
      #ifdef OLIMPO_QL
        Octeto = Car_ConvierteQLaPC ((caracter)Octeto);
      #endif
      CarExt[2] = Digito [Octeto / 16];
      CarExt[3] = Digito [Octeto % 16];
      Fch_EscribeOcteto (Fichero,CarExt,4);
      }
    }
  }

/*--------------------------------------------------------------------
 * FUNCION:  Rtf_NuevaLinea()
 * OBJETIVO: Escribir en un fichero RTF el (los) carcter(es) de
 *           nueva lnea
 * ENTRADAS: El fichero
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void Rtf_NuevaLinea (Fichero)
fichero Fichero;
  {
  static octeto NL[2] = {13, 10};
#ifdef OLIMPO_PC
  Fch_EscribeOcteto (Fichero,NL,2);
#endif
#ifdef OLIMPO_QL
  Fch_EscribeOcteto (Fichero,NL+1,1);
#endif
  }

/*--------------------------------------------------------------------
 * FUNCION:  Opciones()
 * OBJETIVO: Seleccionar las opciones del programa
 * ENTRADAS: Ninguna
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void Opciones()
  {
  enum { SALIDA_ESC, SONIDO, TAMANO, POSICION};

  static cadena MenuOpc[] = {">Sonido", ">Tamao Sub/Sup",
                             ">Posicin Sub/Sup", NIL };

  entero   Opcion;
  contador Fil=4, Col=48, Ancho=31, Alto=4;
  logico   Sigue = SI;

  Cdr_Caja (CDR_SIMPLE,Fil,Col,Fil+Alto,Col+Ancho,NEGRO,VERDE);

  Opcion = SONIDO;
  while ( Sigue )
    {
    Pan_Color (NEGRO,VERDE);

    if ( Son_Estado() ) { Pan_PonTexto (Fil+1,Col+20,"Encendido"); }
    else                { Pan_PonTexto (Fil+1,Col+20,"Apagado  "); }
    Pan_PonEntero (Fil+2,Col+20,TamanoSubSuper,1);
    Pan_PonEntero (Fil+3,Col+20,PosicionSubSuper,1);

    Opcion = Men_Vertical (Fil+1,Col+1,Fil+Alto-1,Col+19,MenuOpc,Opcion);
    switch ( Opcion )
      {
      case SALIDA_ESC:    Sigue = NO;      break;
      case SONIDO:        Son_Cambia();    break;
      case TAMANO:        TamanoSubSuper = PideTamano (TamanoSubSuper);
                          break;
      case POSICION:      PosicionSubSuper = PidePosicion (PosicionSubSuper);
                          break;
      }
    }
  }

/*--------------------------------------------------------------------
 * FUNCION:  PideTamano()
 * OBJETIVO: Pedir al usuario el tamao de sub y superndice
 * ENTRADAS: El tamao propuesto
 * SALIDAS:  El nuevo tamao
 *------------------------------------------------------------------*/
octeto PideTamano (Tamano)
octeto Tamano;
  {
  octeto Respuesta;
  static contador Fil=12, Col=30, Ancho=19, Alto=4;

  Cdr_Caja (CDR_DOBLE,Fil,Col,Fil+Alto,Col+Ancho,NEGRO,VERDE);

  Pan_Color (NEGRO,VERDE);
  Pan_PonTexto (Fil+1,Col+2,"Elige el tamao:");
  Respuesta = Usr_Entero (Tamano, 4, 1, 100, Fil+3, Col+7, NEGRO, BLANCO);
  Pan_Borra (NEGRO,Fil,Col,Fil+Alto,Col+Ancho);

  return ( Respuesta );
  }

/*--------------------------------------------------------------------
 * FUNCION:  PidePosicion()
 * OBJETIVO: Pedir al usuario la posicin de sub y superndice
 * ENTRADAS: La posicin propuesta
 * SALIDAS:  El nuevo tamao
 *------------------------------------------------------------------*/
octeto PidePosicion (Posicion)
octeto Posicion;
  {
  octeto Respuesta;
  static contador Fil=12, Col=29, Ancho=21, Alto=4;

  Cdr_Caja (CDR_DOBLE,Fil,Col,Fil+Alto,Col+Ancho,NEGRO,VERDE);

  Pan_Color (NEGRO,VERDE);
  Pan_PonTexto (Fil+1,Col+2,"Elige la posicin:");
  Respuesta = Usr_Entero (Posicion, 4, 1, 100, Fil+3, Col+7, NEGRO, BLANCO);
  Pan_Borra (NEGRO,Fil,Col,Fil+Alto,Col+Ancho);

  return ( Respuesta );
  }
